// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_crud_bloc/src/domain/entities/query.dart';

/// {@template crud_data_source}
/// A [BaseDataSource] that provides SCRUD operations.
/// {@endtemplate}
abstract class CrudDataSource<Model> extends BaseDataSource {
  /// {@macro crud_data_source}
  const CrudDataSource();

  /// Creates a new [Model] object.
  Future<void> create(Model object, {String? id});

  /// Gets a [Model] object by its [id].
  Future<Model?> get(String id);

  /// Gets all [Model] objects.
  Future<List<Model?>> getAll();

  /// Updates a [Model] object by its [id].
  Future<void> update(
    String id, {
    Model? object,
    Map<String, dynamic>? raw,
  });

  /// Updates all [Model] objects.
  Future<void> updateAll(Map<String, Object?>? data);

  /// Deletes a [Model] object by its [id].
  Future<void> delete(String id);

  /// Deletes all [Model] objects.
  Future<void> deleteAll();

  /// Queries [Model] objects by [conditions].
  Future<List<Model?>> query(List<QueryInterface> conditions);

  /// Streams [Model] objects by [conditions].
  Stream<List<Model?>> stream({
    String? id,
    List<QueryInterface>? conditions,
    bool includeMetadataChanges = false,
  });
}
