// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

class FlatButtonThemeResolver extends ThemeResolver<FlatButtonStyle,
    FlatButtonThemeExtension, ControlState> {
  const FlatButtonThemeResolver({
    required this.customStyleFn,
  });

  @override
  FlatButtonStyle? computeExtensionValueFn(
    BuildContext context,
    FlatButtonThemeExtension? themeExtension, {
    ControlState? extra,
  }) {
    FlatButtonStyle? style;
    switch (extra) {
      case ControlState.disabled:
        style = themeExtension?.disabledStyle;
        break;
      case ControlState.focused:
        style = themeExtension?.focusedStyle;
        break;
      case ControlState.hovered:
        style = themeExtension?.hoveredStyle;
        break;
      case ControlState.tapped:
        style = themeExtension?.tappedStyle;
        break;
      case null:
      case ControlState.normal:
        style = themeExtension?.normalStyle;
        break;
    }

    return style;
  }

  @override
  FlatButtonThemeExtension? getDefaultExtension(BuildContext context) =>
      FlatButtonThemeExtensionDefault.from(Theme.of(context));

  @override
  final FlatButtonStyle? Function(
    BuildContext context, {
    ControlState? extra,
  }) customStyleFn;
}
